@@ -4,22 +4,23 @@
  *	This module implements embedded windows for table widgets.
  *	Much of this code is adapted from tkGrid.c and tkTextWind.c.
  *
- * Copyright (c) 1998 Jeffrey Hobbs
+ * Copyright (c) 1998-2002 Jeffrey Hobbs
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
+ * RCS: @(#) $Id: tkTableWin.c,v 1.9 2002/11/15 22:19:52 cerney Exp $
  */
  
 #include "tkVMacro.h"
 #include "tkTable.h"
 
 static int	StickyParseProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Tk_Window tkwin, Arg value,
-			    char *widgRec, int offset));
+			Tcl_Interp *interp, Tk_Window tkwin,
+			Arg value, char *widgRec, int offset));
 Arg		StickyPrintProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Window tkwin, char *widgRec, int offset,
-			    Tcl_FreeProc **freeProcPtr));
+			Tk_Window tkwin, char *widgRec, int offset,
+			Tcl_FreeProc **freeProcPtr));
 
 static void	EmbWinLostSlaveProc _ANSI_ARGS_((ClientData clientData,
 						Tk_Window tkwin));
@@ -43,7 +44,7 @@ static Tk_GeomMgr tableGeomType = {
 };
 
 /* windows subcommands */
-static char *winCmdNames[] = {
+static CONST84 char *winCmdNames[] = {
     "cget", "configure", "delete", "move", "names", (char *) NULL
 };
 enum winCommand {
@@ -69,8 +70,10 @@ enum winCommand {
  * Done like this to make the command line parsing easy
  */
 
-static Tk_CustomOption stickyOption = {StickyParseProc, StickyPrintProc,
-				       (ClientData) NULL};
+static Tk_CustomOption stickyOption	= { StickyParseProc, StickyPrintProc,
+					    (ClientData) NULL };
+static Tk_CustomOption tagBdOpt		= { TableOptionBdSet, TableOptionBdGet,
+					    (ClientData) BD_TABLE_WIN };
 
 static Tk_ConfigSpec winConfigSpecs[] = {
   {TK_CONFIG_BORDER, "-background", "background", "Background", NULL,
@@ -78,8 +81,9 @@ static Tk_ConfigSpec winConfigSpecs[] = {
    TK_CONFIG_DONT_SET_DEFAULT|TK_CONFIG_NULL_OK },
   {TK_CONFIG_SYNONYM, "-bd", "borderWidth", (char *)NULL, (char *)NULL, 0, 0},
   {TK_CONFIG_SYNONYM, "-bg", "background", (char *)NULL, (char *)NULL, 0, 0},
-  {TK_CONFIG_PIXELS, "-borderwidth", "borderWidth", "BorderWidth", "-1",
-   Tk_Offset(TableEmbWindow, bd), TK_CONFIG_DONT_SET_DEFAULT },
+  {TK_CONFIG_CUSTOM, "-borderwidth", "borderWidth", "BorderWidth", "",
+   0 /* no offset */,
+   TK_CONFIG_DONT_SET_DEFAULT|TK_CONFIG_NULL_OK, &tagBdOpt },
   {TK_CONFIG_STRING, "-create", (char *)NULL, (char *)NULL, (char *)NULL,
    Tk_Offset(TableEmbWindow, create),
    TK_CONFIG_DONT_SET_DEFAULT|TK_CONFIG_NULL_OK },
@@ -160,7 +164,7 @@ StickyParseProc(clientData, interp, tkwin, value, widgRec, offset)
     ClientData clientData;		/* Not used.*/
     Tcl_Interp *interp;			/* Used for reporting errors. */
     Tk_Window tkwin;			/* Window for text widget. */
-    Arg  value;			/* Value of option. */
+    Arg  value;				/* Value of option. */
     char *widgRec;			/* Pointer to TkTextEmbWindow
 					 * structure. */
     int offset;				/* Offset into item (ignored). */
@@ -197,17 +201,16 @@ static TableEmbWindow *
 TableNewEmbWindow(Table *tablePtr)
 {
     TableEmbWindow *ewPtr = (TableEmbWindow *) ckalloc(sizeof(TableEmbWindow));
+    memset((VOID *) ewPtr, 0, sizeof(TableEmbWindow));
+
+    /*
+     * Set the values that aren't 0/NULL by default
+     */
     ewPtr->tablePtr	= tablePtr;
-    ewPtr->tkwin	= NULL;
-    ewPtr->hPtr		= NULL;
-    ewPtr->bg		= NULL;
-    ewPtr->bd		= -1;
-    ewPtr->create	= NULL;
     ewPtr->relief	= -1;
-    ewPtr->sticky	= 0;
-    ewPtr->padX		= 0;
-    ewPtr->padY		= 0;
-    ewPtr->displayed	= 0;
+    ewPtr->padX		= -1;
+    ewPtr->padY		= -1;
+
     return ewPtr;
 }
 
@@ -228,7 +231,6 @@ TableNewEmbWindow(Table *tablePtr)
 static void
 EmbWinCleanup(Table *tablePtr, TableEmbWindow *ewPtr)
 {
-    /* free the options in the widget */
     Tk_FreeOptions(winConfigSpecs, (char *) ewPtr, tablePtr->display, 0);
 }
 
@@ -257,15 +259,26 @@ EmbWinDisplay(Table *tablePtr, Drawable window, TableEmbWindow *ewPtr,
     int diffx=0;	/* Cavity width - slave width. */
     int diffy=0;	/* Cavity hight - slave height. */
     int sticky = ewPtr->sticky;
+    int padx, pady;
 
-    if (ewPtr->bg)		tagPtr->bg = ewPtr->bg;
-    if (ewPtr->relief != -1)	tagPtr->relief = ewPtr->relief;
-    if (ewPtr->bd >= 0)		tagPtr->bd = ewPtr->bd;
+    if (ewPtr->bg)		tagPtr->bg	= ewPtr->bg;
+    if (ewPtr->relief != -1)	tagPtr->relief	= ewPtr->relief;
+    if (ewPtr->borders) {
+	tagPtr->borderStr	= ewPtr->borderStr;
+	tagPtr->borders		= ewPtr->borders;
+	tagPtr->bd[0]		= ewPtr->bd[0];
+	tagPtr->bd[1]		= ewPtr->bd[1];
+	tagPtr->bd[2]		= ewPtr->bd[2];
+	tagPtr->bd[3]		= ewPtr->bd[3];
+    }
 
-    x += ewPtr->padX/2;
-    width -= ewPtr->padX;
-    y += ewPtr->padY/2;
-    height -= ewPtr->padY;
+    padx = (ewPtr->padX < 0) ? tablePtr->padX : ewPtr->padX;
+    pady = (ewPtr->padY < 0) ? tablePtr->padY : ewPtr->padY;
+
+    x		+= padx;
+    width	-= padx*2;
+    y		+= pady;
+    height	-= pady*2;
 
     if (width > Tk_ReqWidth(ewPtr->tkwin)) {
 	diffx = width - Tk_ReqWidth(ewPtr->tkwin);
@@ -288,8 +301,10 @@ EmbWinDisplay(Table *tablePtr, Drawable window, TableEmbWindow *ewPtr,
 	y += (sticky&STICK_SOUTH) ? diffy : diffy/2;
     }
 
-    /* If we fall below a specific minimum width/height requirement,
-   * we just unmap the window */
+    /*
+     * If we fall below a specific minimum width/height requirement,
+     * we just unmap the window
+     */
     if (width < 4 || height < 4) {
 	if (ewPtr->displayed) {
 	    EmbWinUnmapNow(ewTkwin, tkwin);
@@ -359,7 +374,9 @@ EmbWinUnmap(Table *tablePtr, int rlo, int rhi, int clo, int chi)
     int row, col, trow, tcol;
     char buf[INDEX_BUFSIZE];
 
-    /* we need to deal with things user coords */
+    /*
+     * Transform numbers from real to user user coords
+     */
     rlo += tablePtr->rowOffset;
     rhi += tablePtr->rowOffset;
     clo += tablePtr->colOffset;
@@ -408,7 +425,9 @@ EmbWinRequestProc(clientData, tkwin)
 {
     register TableEmbWindow *ewPtr = (TableEmbWindow *) clientData;
 
-  /* resize depends on the sticky */
+    /*
+     * Resize depends on the sticky
+     */
     if (ewPtr->displayed && ewPtr->hPtr != NULL) {
 	Table *tablePtr = ewPtr->tablePtr;
 	int row, col, x, y, width, height;
@@ -595,6 +614,7 @@ EmbWinConfigure(tablePtr, ewPtr, objc, objv)
      int objc;			/* Number of objs in objv. */
      Tcl_Obj *CONST objv[];	/* Obj type options. */
 {
+    Tcl_Interp *interp = tablePtr->interp;
     Tk_Window oldWindow;
     int i, result;
     Arg *args;
@@ -606,7 +626,7 @@ EmbWinConfigure(tablePtr, ewPtr, objc, objv)
     for (i = 0; i < objc; i++)
 	args[i] = LangStringArg(Tcl_GetString(objv[i]));
     args[i] = NULL;
-    result = Tk_ConfigureWidget(tablePtr->interp, tablePtr->tkwin,
+    result = Tk_ConfigureWidget(interp, tablePtr->tkwin,
 				winConfigSpecs, objc, args, (char *) ewPtr,
 				TK_CONFIG_ARGV_ONLY);
     ckfree((char *) args);
@@ -641,7 +661,7 @@ EmbWinConfigure(tablePtr, ewPtr, objc, objv)
 		}
 		if (Tk_IsTopLevel(ancestor)) {
 		badMaster:
-		    Tcl_AppendStringsToObj(Tcl_GetObjResult(tablePtr->interp),
+		    Tcl_AppendStringsToObj(Tcl_GetObjResult(interp),
 					   "can't embed ",
 					   Tk_PathName(ewPtr->tkwin), " in ",
 					   Tk_PathName(tablePtr->tkwin),
